Imports ProgressBar.Framework.Abstractions

Namespace ProgressBar.Framework
	''' <summary>
	''' Server-side API for the SimpleProgress framework
	''' </summary>
	Public Class ProgressManager
		Implements IProgressManager
		''' <summary>
		''' Header through which the ID that uniquely identifies the remote task is identified.
		''' </summary>
		Public Const HeaderNameTaskId As String = "X-ProgressBar-TaskId"

		Private ReadOnly _dataProvider As IProgressDataProvider

		''' <summary>
		''' Builds the root of the framework passing the component responsible for storing 
		''' the status of each request. By default, status is stored in the ASP.NET cache.
		''' </summary>
		Public Sub New()
			Me.New(New AspnetProgressProvider)
		End Sub
		Public Sub New(ByVal dataProvider As IProgressDataProvider)
			_dataProvider = dataProvider
		End Sub

		''' <summary>
		''' Used by controllers (or worker services) to set the percentage of work completed.
		''' </summary>
		''' <param name="taskId">ID of the current task being monitored</param>
		''' <param name="format">Format of the string to output</param>
		''' <param name="args">Arguments to embed in the string</param>
		Public Sub SetCompleted(ByVal taskId As String, ByVal format As String, ParamArray ByVal args() As Object) Implements IProgressManager.SetCompleted
			If String.IsNullOrEmpty(taskId) Then
                Exit Sub
			End If
			Dim [step] = String.Format(format, args)
			_dataProvider.Set(taskId, [step])
		End Sub

		''' <summary>
		''' Used by controllers (or worker services) to set the percentage of work completed.
		''' </summary>
		''' <param name="taskId">ID of the current task being monitored</param>
		''' <param name="percentage">Percentage of the work accomplished</param>
		Public Sub SetCompleted(ByVal taskId As String, ByVal percentage As Int32) Implements IProgressManager.SetCompleted
			percentage = If(percentage < 0, 0, percentage)
			percentage = If(percentage > 100, 100, percentage)

			SetCompleted(taskId, "{0} %", percentage)
		End Sub

		''' <summary>
		''' Used by controllers (or worker services) to set a description of the current task. 
		''' </summary>
		''' <param name="taskId">ID of the current task being monitored</param>
		''' <param name="step">Description of the current stage</param>
		Public Sub SetCompleted(ByVal taskId As String, ByVal [step] As String) Implements IProgressManager.SetCompleted
			If String.IsNullOrEmpty(taskId) Then
                Exit Sub
			End If

			_dataProvider.Set(taskId, [step])
		End Sub

		''' <summary>
		''' Returns the status of the current task.
		''' </summary>
		''' <param name="taskId">ID of the current task being monitored</param>
		''' <returns>Current status of the task</returns>
		Public Function GetStatus(ByVal taskId As String) As String Implements IProgressManager.GetStatus
			Const invalidStatus As String = "..."
			If String.IsNullOrEmpty(taskId) Then
				Return invalidStatus
			End If

			Dim buffer = _dataProvider.Get(taskId)
			Return If(String.IsNullOrEmpty(buffer), invalidStatus, buffer)
		End Function

		''' <summary>
		''' Queues a message for the remote task to terminate
		''' </summary>
		''' <param name="taskId">ID of the current task being monitored</param>
		Public Sub RequestTermination(ByVal taskId As String) Implements IProgressManager.RequestTermination
			If String.IsNullOrEmpty(taskId) Then
                Exit Sub
			End If
			_dataProvider.Abort(taskId)
		End Sub

		''' <summary>
		''' Reads whether the task should terminate
		''' </summary>
		''' <param name="taskId">ID of the current task being monitored</param>
		''' <returns>True if the task should be interrupted, false otherwise</returns>
		Public Function ShouldTerminate(ByVal taskId As String) As Boolean Implements IProgressManager.ShouldTerminate
			Return _dataProvider.ShouldTerminate(taskId)
		End Function
	End Class
End Namespace